/*
 * #%L
 * *********************************************************************************************************************
 * 
 * SolidBlue - open source safe data
 * http://solidblue.tidalwave.it - hg clone https://bitbucket.org/tidalwave/solidblue-src
 * %%
 * Copyright (C) 2011 - 2013 Tidalwave s.a.s. (http://tidalwave.it)
 * %%
 * *********************************************************************************************************************
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * *********************************************************************************************************************
 * 
 * $Id$
 * 
 * *********************************************************************************************************************
 * #L%
 */
package it.tidalwave.integritychecker.archive;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.nio.file.Path;
import org.joda.time.DateTime;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.ToString;
import lombok.experimental.Wither;

/***********************************************************************************************************************
 *
 * A record relative to a scan for given file at a given time.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Immutable @Getter @RequiredArgsConstructor @EqualsAndHashCode @ToString 
public class ScanRecord 
  {
    @RequiredArgsConstructor @Immutable @Wither
    public static class Builder
      {
        public static interface Callback // Lombok @Wither doesn't support builder subclasses
          {
            public void register (final @Nonnull ScanRecord scanRecord);

            public static final Callback DEFAULT = new Callback()
              {
                @Override
                public void register (final @Nonnull ScanRecord scanRecord) 
                  {
                  }
              };
          }
        
        private final Path file;

        private final String signature;

        private final DateTime scanDateTime;
        
        private final Callback callback;

        public Builder() 
          {
            this(null, null, null, Callback.DEFAULT);
          }
        
        public Builder (final @Nonnull Callback callback) 
          {
            this(null, null, null, callback);
          }
        
        @Nonnull
        public ScanRecord build()
          {
            final ScanRecord scanRecord = new ScanRecord(file, signature, scanDateTime);
            callback.register(scanRecord);
            return scanRecord;
          }
      }
    
    @Nonnull
    private final Path file;
    
    @Nonnull
    private final String signature;
    
    @Nonnull
    private final DateTime scanDateTime;
  }
